from functools import partial
from typing import get_type_hints, Type

from fastapi import APIRouter
from starlette.requests import Request as StarletteRequest

from nexium_api.utils.api_error import APIError
from nexium_api.request.base_auth import BaseRequestAuth
from nexium_api.request.process_request import process_request
from nexium_api.request.request import Request as BaseRequest
from nexium_api.response.response import Response as BaseResponse


class BaseRouter:
    prefix: str = ''
    request_auth = BaseRequestAuth

    def __init__(self, prefix: str = '', auth: BaseRequestAuth = None, errors: list[Type[APIError]] = None):
        # For API Client
        self.auth = auth
        self.errors = errors

        # Tag
        tag = type(self).__mro__[0].__name__
        if tag.endswith('Router'):
            tag = tag[:-6]

        self.fastapi = APIRouter(
            prefix=self.prefix,
            tags=[tag],
        )
        self.prefix = prefix + self.prefix

        # Init child Routers
        for attr_name, attr in get_type_hints(self.__class__).items():
            if not issubclass(attr, BaseRouter):
               continue

            child_router = attr(
                prefix=self.prefix,
                auth=self.auth,
                errors=self.errors,
            )
            setattr(self, attr_name, child_router)
            self.fastapi.include_router(child_router.fastapi)

        # Child routes
        # noinspection PyUnresolvedReferences
        self.routes = [
            self.__getattribute__(name)
            for name in dir(self)
            if hasattr(self.__getattribute__(name), '__wrapped__') and self.__getattribute__(name).__name__
        ]

        # Init child routes
        for route in self.routes:
            path, type_, func, request_data, response_data, request_auth, check_request_auth, kwargs = route.params
            request_auth = request_auth if request_auth else self.request_auth

            class Request(BaseRequest):
                auth: request_auth
                data: request_data

            class Response(BaseResponse):
                data: response_data

            async def route_handler(request: Request, starlette_request: StarletteRequest, func):
                return await process_request(
                    request=request,
                    starlette_request=starlette_request,
                    func=func,
                )

            self.fastapi.post(
                path=path,
                response_model=Response,
                **kwargs,
            )(partial(route_handler, func=func))
